import { SQS } from 'aws-sdk';
import { Consumer } from 'sqs-consumer';
import { DefaultTaskContext, GetContextFn, OperationConfiguration, OperationName, QueueConfiguration, QueueName } from './types';
export interface ClientConfiguration<TContext> {
    defaultQueue: QueueConfiguration;
    /**
     * Async method invoked for each message that arrives to generate a context
     *
     * Defaults to a method that returns DefaultTaskContext
     */
    getContext?: GetContextFn<TContext>;
    /**
     * Optionally specify additional queues aside from the requisite default queue
     */
    queues?: Record<QueueName, QueueConfiguration>;
    /**
     * The SQS client provided to queues upon initialization
     *
     * Defaults to `new AWS.SQS()`
     */
    sqsClient?: SQS;
}
export declare type RegisterOperationInput<T> = OperationConfiguration<T>;
export interface SubmitTaskInput<T> {
    operationName: string;
    payload: T;
}
export interface SubmitTaskResponse {
    messageId: string | null;
    taskId: string;
}
/**
 * Handles configuring queues, registering operations, and enqueueing/dequeueing tasks
 * for processing
 */
export declare class AsyncTasksClient<TContext = DefaultTaskContext> {
    private sqsClient;
    private queues;
    private routes;
    private getContext;
    private consumers;
    constructor(config: ClientConfiguration<TContext>);
    readonly registeredOperations: OperationName[];
    /**
     * Register an operation type with the asyncTask client
     *
     * Operations must have a name, a validation function, and a handler function and may
     * specify a queue on which jobs should be submitted by default
     *
     * @param input
     * @throws {QueueNotRegistered} the queueId specified is not registered
     */
    registerOperation<T>(input: RegisterOperationInput<T>): void;
    /**
     * Enqueues a task for a specified operation
     *
     * * Tasks must specify an operationName and a payload that can be serialized to JSON.
     * * The payload will be validated against the validation function provided when the
     *   operation was registered
     *
     * @param input
     * @throws {OperationNotRegistered} the operationName has not been configured
     * @throws {InvalidPayloadError} the provided payload did not pass validation
     * @throws {QueueNotRegistered} the specified queue is not configured
     * @returns the SQS MessageId and a unique taskId generated on our side
     */
    submitTask<T>(input: SubmitTaskInput<T>): Promise<SubmitTaskResponse>;
    getConsumerInstances(): Record<QueueName, Consumer>;
    private _createQueueConsumers;
}
